module MdToHtmlTest where

import MdToHTML
import Test.HUnit

check_equal :: String -> String -> String -> Test
check_equal desc expected actual = TestCase (assertEqual desc expected actual)

convert :: String -> String
convert md = show . fst $ leftmostLongestParse parseDocument md

headerTests =
  TestList
    [ check_equal "Should convert H1 heading" "<h1>Hello</h1>" (convert "# Hello"),
      check_equal "Should convert H2 heading" "<h2>Hello</h2>" (convert "## Hello"),
      check_equal "Should convert H3 heading" "<h3>Hello</h3>" (convert "### Hello"),
      check_equal "Should convert H4 heading" "<h4>Hello</h4>" (convert "#### Hello"),
      check_equal "Should convert H5 heading" "<h5>Hello</h5>" (convert "##### Hello"),
      check_equal "Should convert H6 heading" "<h6>Hello</h6>" (convert "###### Hello")
    ]

boldTests =
  TestList
    [ check_equal "Should convert bold" "<p><b>Hello</b></p>" (convert "__Hello__"),
      check_equal "Should convert italic" "<p><i>Hello</i></p>" (convert "_Hello_"),
      check_equal "Should convert bold and italic in a sentence" "<p>It <i>is</i> a <b>wonderful</b> day</p>" (convert "It _is_ a __wonderful__ day"),
      check_equal "Should convert nested bold and italic" "<p><b>Bold then <i>Italic</i></b></p>" (convert "**Bold then *Italic***"),
      check_equal "Should convert nested bold and italic" "<p><i>Italic then <b>Bold</b></i></p>" (convert "*Italic then **Bold***")
    ]

strikethroughTests =
  TestList
    [ check_equal "Should convert strikethrough" "<p><s>Hello</s></p>" (convert "~~Hello~~"),
      check_equal "Should convert long sentence with tilde" "<p><s>The universe is ~7 days old</s>. The universe is 13 billion years old.</p>" (convert "~~The universe is ~7 days old~~. The universe is 13 billion years old.")
    ]

linkTests =
  TestList
    [ check_equal "Should convert normal link" "<p><a href=\"https://example.com\">This is an example link.</a></p>" (convert "[This is an example link.](https://example.com)"),
      check_equal "Should convert styled link" "<p><a href=\"https://example.com\"><b>Fancy</b>!!!</a></p>" (convert "[__Fancy__!!!](https://example.com)")
    ]

escapedCharTests =
  TestList
    [ check_equal "Should print literal underscore" "<p>This is an underscore - _</p>" (convert "This is an underscore - \\_"),
      check_equal "Should print literal asterisk" "<p>This is an asterisk - *</p>" (convert "This is an asterisk - \\*"),
      check_equal "Should print literal asterisk in bold" "<p>This is a bolded asterisk - <b>*</b></p>" (convert "This is a bolded asterisk - **\\***")
    ]

blockquoteTests =
  TestList
    [ check_equal "Should wrap para in blockquote" "<blockquote><p>What a <b>truly</b> <i>lovely</i> day!!!</p></blockquote>" (convert "> What a __truly__ _lovely_ day!!!"),
      check_equal "Simple nested blockquotes" "<blockquote><p>Hello</p><blockquote><p>World</p></blockquote></blockquote>" (convert "> Hello\n>\n>> World"),
      check_equal
        "Nested blockquotes"
        "<blockquote><p>Dorothy followed her through many \
        \of the beautiful rooms in her castle.</p><blockquote><p>The Witch \
        \bade her clean the pots and kettles and sweep the floor and keep the fire \
        \fed with wood.</p></blockquote></blockquote>"
        ( convert
            "> Dorothy followed her through many of the \
            \beautiful rooms in her castle.\n> \n>> The Witch bade her \
            \clean the pots and kettles and sweep the floor and keep the fire fed with wood."
        )
    ]

unorderedListTests =
  TestList
    [ check_equal "Basic ordered list" "<ul><li>Item 1</li><li>Item 2</li><li>Item 3</li></ul>" (convert "* Item 1\n* Item 2\n* Item 3"),
      check_equal "Mixing list indicators" "<ul><li>Item 1</li><li>Item 2</li><li>Item 3</li></ul>" (convert "* Item 1\n+ Item 2\n- Item 3"),
      check_equal "Formatted lists" "<ul><li><b>Item 1</b></li><li><i>Item 2</i></li><li><b><i>Item 3</i></b></li></ul>" (convert "* __Item 1__\n+ _Item 2_\n- ***Item 3***"),
      check_equal "Nested list" "<ul><li>Item 1</li><li>Item 2</li><li>Item 3<ul><li>Subitem 1</li><li>Subitem 2</li></ul></li></ul>" (convert "* Item 1\n* Item 2\n* Item 3\n    * Subitem 1\n    * Subitem 2"),
      check_equal "Paragraph in list" "<ul><li>Item 1</li><li><p>Item 2</p><p>More stuff</p></li><li>Item 3</li></ul>" (convert "- Item 1\n- Item 2\n\n    More stuff\n\n- Item 3"),
      check_equal "Paragraph before list" "<p>This is a list</p><ul><li>Item 1</li><li>Item 2</li></ul>" (convert "This is a list\n\n* Item 1\n* Item 2"),
      check_equal "Paragraph before list" "<h3>This is a list</h3><ul><li>Item 1</li><li>Item 2</li></ul>" (convert "### This is a list\n\n* Item 1\n* Item 2"),
      check_equal "Nested list then back" "<ul><li>Item 1</li><li>Item 2<ul><li>Item 3</li><li>Item 4</li></ul></li><li>Item 5</li></ul>" (convert "- Item 1\n- Item 2\n    - Item 3\n    - Item 4\n- Item 5")
    ]

integrationTests =
  TestList
    [ check_equal "Integration 1" "<h1>Sample Markdown</h1><p>This is some basic, sample markdown.</p><h2><b>Second</b> <i>Heading</i></h2>" (convert "# Sample Markdown\n\n This is some basic, sample markdown.\n\n  ## __Second__ _Heading_"),
      check_equal "Integration 2" "<p><b>Hello</b> <i>World</i></p>" (convert "__Hello__\n_World_"),
      check_equal "Integration 3" "<h1>Hello</h1><p>World</p>" (convert "# Hello\nWorld"),
      check_equal "Integration 4" "<p>a b</p>" (convert "a\nb"),
      check_equal "Integration 5" "<h1>Hello</h1>" (convert "# Hello\n"),
      check_equal "Integration 6" "<p>First line<br>Second line</p>" (convert "First line  \nSecond line"),
      check_equal
        "Integration 7"
        "<h1>Sample Markdown</h1><p>This is some basic, sample markdown.</p><h2>Second \
        \Heading</h2><ul><li>Unordered lists, and:<ul><li>One</li><li>Two</li><li>\
        \Three</li></ul></li><li>More</li></ul><blockquote><p>Blockquote</p>\
        \</blockquote><p>And <b>bold</b>, <i>italics</i>, and even <i>italics \
        \and later <b>bold</b></i>. Even <s>strikethrough</s>. \
        \<a href=\"https://markdowntohtml.com\">A link</a> to somewhere.</p>"
        ( convert
            "# Sample Markdown\n\nThis is some basic, sample markdown.\n\n## Second \
            \Heading\n\n- Unordered lists, and:\n    - One\n    - Two\n    - Three\n\
            \- More\n\n> Blockquote\n\nAnd **bold**, *italics*, and even *italics and \
            \later **bold***. Even ~~strikethrough~~. [A link](https://markdowntohtml.com) to somewhere."
        )
    ]

tests =
  TestList
    [ headerTests,
      boldTests,
      strikethroughTests,
      linkTests,
      escapedCharTests,
      blockquoteTests,
      unorderedListTests,
      integrationTests
    ]

runTests = runTestTT tests
